from tornado.web import RequestHandler

from blueshed.micro.web.user_mixin import UserMixin
from blueshed.micro.web.cors_mixin import CorsMixin, cors


class RemoteSignOutHandler(UserMixin, CorsMixin, RequestHandler):
    ''' Removes the cookie from application settings. '''

    def initialize(self, http_origins=None):
        self.set_cors_methods('GET,OPTIONS')

        if http_origins:
            self.set_cors_whitelist(http_origins)

    def options(self):
        return self.cors_options()

    @cors
    def get(self):
        ''' removes cookie and responds to confirm '''
        self.clear_cookie(self.cookie_name)
        self.write({"result": "OK"})



import json
from urllib.parse import urlencode

from tornado.httpclient import AsyncHTTPClient
from tornado.httpclient import HTTPError


async def get_user(user_id, *, base_url, service_name, service_secret):
    client = AsyncHTTPClient()

    credientials = {
        'service-name': service_name,
        'service-secret': service_secret}

    query = urlencode({'id': user_id})
    url   = '{base_url}/get_user'.format(base_url=base_url)

    try:
        response = await client.fetch(
            url,
            method='POST',
            headers=credientials,
            body=query)
    except HTTPError:
        raise
    else:
        return json.loads(response.body.decode('utf-8'))['result']



from urllib.parse import urlencode

from tornado.httpclient import AsyncHTTPClient
from tornado.web import RequestHandler
from tornado.web import HTTPError as ServerHTTPError
from tornado.httpclient import HTTPError as ClientHTTPError
from tornado.escape import json_decode
from tornado import gen

from blueshed.micro.web.user_mixin import UserMixin
from blueshed.micro.web.cors_mixin import CorsMixin, cors


class RemoteSignInHandler(UserMixin, CorsMixin, RequestHandler):
    '''
        Validates a one-time-password with an auth server and
        puts the result into our access control cookie
        and redirects.
    '''

    def initialize(self, service_name, service_secret, auth_url, http_origins=None):  # noqa
        self.service_name = service_name
        self.service_secret = service_secret
        self.auth_url = auth_url

        # cors options
        self.set_cors_methods('GET,OPTIONS')
        if http_origins:
            self.set_cors_whitelist(http_origins)

    def write_error(self, *args, **kwargs):
        '''
        Must override base write error to stop uncaught
        HTTP errors from clearing CORS headers
        '''
        self.write_cors_headers()
        RequestHandler.write_error(self, *args, **kwargs)

    def options(self):
        return self.cors_options()

    @property
    def auth_headers(self):
        return {
            "service-name": self.service_name,
            "service-secret": self.service_secret
        }

    @cors
    @gen.coroutine
    def get(self):
        # get otp from request
        otp = self.get_argument("otp")

        try:
            # make request to auth server
            response = yield AsyncHTTPClient().fetch(
                self.auth_url + '/control/get_user',
                method="POST",
                headers=self.auth_headers,
                body=urlencode({"user_otp": otp})
            )
        except ClientHTTPError as error:
            # check content type for json to try and parse result
            if error.response and error.response.headers.get('Content-Type') is 'application/json; charset=UTF-8':
                # decode the response
                content = json_decode(error.response.body)

                # catch client otp errors
                if content['status_code'] == 400:
                    raise ServerHTTPError(401, reason=content['message'])

            # handle all other cases as internal errors
            raise error

        else:
            # set client cookie and respond
            content = json_decode(response.body)
            self.set_secure_cookie(self.cookie_name, str(content['result']['id']))
            self.write({"result": "OK"})
            self.finish()



from distutils.core import setup
from setuptools import find_packages
from pip.req import parse_requirements as parse


setup(
    name='talka2z',
    version='0.4',
    packages=find_packages(exclude=('tests', 'tests.*')),
    install_requires=[
    ]
)
